/**
* @file tkl_system.h
* @brief Common process - adpater some api which provide system
* @version 0.1
* @date 2020-11-09
*
* @copyright Copyright 2020-2021 Tuya Inc. All Rights Reserved.
*
*/
#ifndef __TUYA_HAL_SYSTEM_H__
#define __TUYA_HAL_SYSTEM_H__

#include "tuya_cloud_types.h"

#ifdef __cplusplus
extern "C" {
#endif

#if OPERATING_SYSTEM == SYSTEM_LINUX
typedef UINT64_T TKL_SYS_TICK_T;
typedef UINT64_T TKL_SYS_TIME_T;
#else
typedef UINT_T TKL_SYS_TICK_T;
typedef UINT_T TKL_SYS_TIME_T;
#endif

/*
 *  reasons for restart
 */
typedef enum {
    TKL_RESET_REASON_UNKNOWN    = 0, ///< Underterminable cause
    TKL_RESET_REASON_FIB        = 1, ///< Reset originated from the FIB bootloader
    TKL_RESET_REASON_BOOTLOADER = 2, ///< Reset relates to an Ember bootloader
    TKL_RESET_REASON_EXTERNAL   = 3, ///< External reset trigger
    TKL_RESET_REASON_POWERON    = 4, ///< Poweron reset type, supply voltage < power-on threshold
    TKL_RESET_REASON_WATCHDOG   = 5, ///< Watchdog reset occurred
    TKL_RESET_REASON_SOFTWARE   = 6, ///< Software triggered reset
    TKL_RESET_REASON_CRASH      = 7, ///< Software crash
    TKL_RESET_REASON_FLASH      = 8, ///< Flash failure cause reset
    TKL_RESET_REASON_FATAL      = 9, ///< A non-recoverable fatal error occurred
    TKL_RESET_REASON_FAULT      = 10,///< A access fault occurred
    TKL_RESET_REASON_BROWNOUT   = 11,///< Brown out

    TKL_RESET_REASON_DEEPSLEEP  = 12,///< Reset caused by entering deep sleep 
    TKL_RESET_REASON_UNSUPPORT  = 0xFF,
} TKL_RESET_REASON_E;

/**
 * @brief system enter critical
 *
 * @param[in]   none
 * @return  irq mask
 */
UINT_T tkl_system_enter_critical(VOID_T);

/**
 * @brief system exit critical
 *
 * @param[in]   irq_mask: irq mask 
 * @return  none
 */
VOID_T tkl_system_exit_critical(UINT_T irq_mask);

/**
 * @brief enter critical macro
 */
#define TKL_ENTER_CRITICAL()        \
    UINT_T __irq_mask;              \
    __irq_mask = tkl_system_enter_critical()

/**
 * @brief exit critical macro
 */
#define TKL_EXIT_CRITICAL()         \
    tkl_system_exit_critical(__irq_mask)



/**
* @brief system reset
*
* @param none
*
* @return none
*/
VOID_T tkl_system_reset(VOID_T);

/**
* @brief Get system tick count
*
* @param none
*
* @return system tick count
*/
TKL_SYS_TICK_T tkl_system_get_tick_count(VOID_T);

/**
* @brief Get system millisecond
*
* @param none
*
* @return system millisecond
*/
TKL_SYS_TIME_T tkl_system_get_millisecond(VOID_T);

/**
* @brief Get system random data
*
* @param[in] range: random from 0  to range
*
* @return random value
*/
INT_T tkl_system_get_random(UINT_T range);

/**
* @brief Get system reset reason
*
* @param[in] describe: point to reset reason describe
*
* @return reset reason
*/
TKL_RESET_REASON_E tkl_system_get_reset_reason(CHAR_T** describe);

/**
* @brief  system sleep
*
* @param[in] describe: num ms
*
* @return none
*/
VOID_T tkl_system_sleep(UINT_T num_ms);


/**
* @brief system delay
*
* @param[in] msTime: time in MS
*
* @note This API is used for system sleep.
*
* @return VOID
*/
VOID_T tkl_system_delay(UINT_T num_ms);




#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif

