/*********************************************************************************
  *Copyright(C),2015-2020, 
  *TUYA 
  *www.tuya.com
  *FileName:    tuya_ipc_speaker_demo.c
**********************************************************************************/
#include "tuya_ipc_common_demo.h"
#if defined(DEMO_IPC_VOICE_ENABLE) && (DEMO_IPC_VOICE_ENABLE == 1)
#include <stdio.h>
#include <string.h>
#include <pthread.h>
#include <sys/prctl.h>
#include "tuya_ipc_media_wave_parse.h"
#include "tuya_ipc_alsa_play_demo.h"
#include "tuya_ipc_speaker_demo.h"

typedef struct ty_speaker_handle {
    IPC_SPEAKER_STATUS_E    status;
    INT_T                   pause;
    INT_T                   stop;
    pthread_t               stid;
    pthread_mutex_t         lock;
    CHAR_T                  filename[IPC_WAV_FILENAME_LEN+1];
} ty_speaker_handle_s;

STATIC ty_speaker_handle_s *ty_speaker_get_handle()
{
    static ty_speaker_handle_s s_speaker_hdl = {.lock = PTHREAD_MUTEX_INITIALIZER};

    return &s_speaker_hdl;
}

/* set speaker status */
STATIC VOID ty_speaker_set_status(IPC_SPEAKER_STATUS_E status)
{
    ty_speaker_handle_s* phdl = ty_speaker_get_handle();

    pthread_mutex_lock(&phdl->lock);
    phdl->status = status;
    pthread_mutex_unlock(&phdl->lock);
}

/* get speaker status */
IPC_SPEAKER_STATUS_E ty_speaker_get_status()
{
    IPC_SPEAKER_STATUS_E status;
    ty_speaker_handle_s* phdl = ty_speaker_get_handle();

    pthread_mutex_lock(&phdl->lock);
    status = phdl->status;
    pthread_mutex_unlock(&phdl->lock);

    return status;
}

/* set speaker pause state: 1 => pause, 0 => resume play */
VOID ty_speaker_set_pause(INT_T pause)
{
    ty_speaker_handle_s* phdl = ty_speaker_get_handle();
    
    pthread_mutex_lock(&phdl->lock);
    phdl->pause = pause;
    pthread_mutex_unlock(&phdl->lock);
}

/* get speaker pause state */
INT_T ty_speaker_get_pause()
{
    INT_T pause;
    ty_speaker_handle_s* phdl = ty_speaker_get_handle();

    pthread_mutex_lock(&phdl->lock);
    pause = phdl->pause;
    pthread_mutex_unlock(&phdl->lock);

    return pause;
}

/* set speaker stop state */
VOID ty_speaker_set_stop(INT_T stop)
{
    ty_speaker_handle_s* phdl = ty_speaker_get_handle();
    
    pthread_mutex_lock(&phdl->lock);
    phdl->stop = stop;
    pthread_mutex_unlock(&phdl->lock);
}

/* get speaker stop state */
INT_T ty_speaker_get_stop()
{
    INT_T stop;
    ty_speaker_handle_s* phdl = ty_speaker_get_handle();

    pthread_mutex_lock(&phdl->lock);
    stop = phdl->stop;
    pthread_mutex_unlock(&phdl->lock);

    return stop;
}

/* speaker play thread */
static void *ty_speaker_proc(void *arg)
{
    INT_T left_size = 0;
    INT_T buffer_size = 0;
    INT_T tms = 0;
    CHAR_T *buffer = NULL;
    IPC_SPEAKER_STATUS_E status;
    ty_media_wav_s *pwav = NULL;
    ty_alsa_play_handle_s *play_handle = NULL;
    ty_speaker_handle_s* phdl = ty_speaker_get_handle();

    PR_DEBUG("speaker thread begin__");

    prctl(PR_SET_NAME, "speaker");

    /* open wav file */
    pwav = ty_media_wav_open(phdl->filename);
    if (pwav == NULL)
    {
        PR_ERR("open media wav %s failed", phdl->filename);
        status = IPC_SPEAKER_STATUS_ERR;
        goto leave;
    }
    fseek(pwav->fp, pwav->data_offset, SEEK_SET);
    left_size = pwav->data_size;

    /* init alsa interface */
    /* if user not use alsa interface, please use other audio interface init code instead. */
    play_handle = ty_alsa_play_init(pwav);
    if (play_handle == NULL)
    {
        PR_ERR("alsa play init failed");
        status = IPC_SPEAKER_STATUS_ERR;
        goto leave;
    }

    /* malloc buffer */
    /* for alsa interface, alsa interface need period_data_size pcm data each write operation */
    /* if user not use alsa interface, please use other size based on audio interface. */
    buffer_size = play_handle->period_data_size;
    buffer = (CHAR_T *)malloc(buffer_size);
    if (buffer == NULL)
    {
        PR_ERR("malloc audio play buffer failed");
        status = IPC_SPEAKER_STATUS_ERR;
        goto leave;
    }

    /* calculate sleep time after each write operation */
    tms = buffer_size * 1000 / pwav->format.avg_bytes_per_sec;
    tms = tms / 2;
    printf("tms = %d\n", tms);
    /* play audio */
    while (left_size > 0)
    {
        /* stop playing */
        INT_T stop = ty_speaker_get_stop();
        if (stop)
        {
            status = IPC_SPEAKER_STATUS_IDLE;
            goto leave;
        }
    
        /* pause playing */
        INT_T pause = ty_speaker_get_pause();
        if (pause)
        {
            usleep(tms * 1000);
            ty_speaker_set_status(IPC_SPEAKER_STATUS_PAUSING);
            continue;
        }

        ty_speaker_set_status(IPC_SPEAKER_STATUS_PLAYING);

        /* read data from file */
        size_t ret = fread(buffer, 1, buffer_size, pwav->fp);
        if (ret > 0)
        {
            /* play data by alsa interface */
            if (ty_alsa_play_data(play_handle, buffer, ret) < 0)
            {
                PR_ERR("alsa play failed");
                status = IPC_SPEAKER_STATUS_ERR;
                goto leave;
            }
            left_size -= ret;
        }
        else
        {
            PR_ERR("read file %s failed", phdl->filename);
            status = IPC_SPEAKER_STATUS_ERR;
            goto leave;
        }
        usleep(tms * 1000);
    }
    status = IPC_SPEAKER_STATUS_IDLE;

leave:
    if (buffer != NULL)
    {
        free(buffer);
    }
    ty_alsa_play_uninit(&play_handle);
    ty_media_wav_close(&pwav);
    ty_speaker_set_status(status);
    PR_DEBUG("speaker thread end__");
}

/* play wav file */
INT_T ty_speaker_play_file(CHAR_T *filename)
{
    INT_T ret;
    IPC_SPEAKER_STATUS_E status;

    if (filename == NULL)
    {
        return -1;
    }

    ty_speaker_handle_s* phdl = ty_speaker_get_handle();

    /* continue playing if the file is paused */
    status = ty_speaker_get_status();
    if ((0 == strcmp(filename, phdl->filename)) && (status == IPC_SPEAKER_STATUS_PAUSING))
    {
        PR_DEBUG("continue playing...");
        ty_speaker_set_stop(0);
        ty_speaker_set_pause(0);
        return 0;
    }

    /* clear status */
    if (status == IPC_SPEAKER_STATUS_ERR)
    {
        status = IPC_SPEAKER_STATUS_IDLE;
        ty_speaker_set_status(status);
    }
    
    if (status != IPC_SPEAKER_STATUS_IDLE)
    {
        PR_ERR("speaker thread have start...");
        return -1;
    }

    ty_speaker_set_stop(0);
    ty_speaker_set_pause(0);
    strncpy(phdl->filename, filename, sizeof(phdl->filename) - 1);

    pthread_attr_t attr;
    ret = pthread_attr_init(&attr);
    if (ret != 0)
    {
        PR_ERR("pthread_attr_init failed ret[%d]", ret);
        ty_speaker_set_status(IPC_SPEAKER_STATUS_ERR);
        return ret;
    }
    pthread_attr_setstacksize(&attr, 64 * 1024);
    ret = pthread_create(&phdl->stid, &attr, ty_speaker_proc, NULL);
    pthread_attr_destroy(&attr);
    if (ret != 0)
    {
        PR_ERR("speaker thread create failed ret[%d]", ret);
        ty_speaker_set_status(IPC_SPEAKER_STATUS_ERR);
        return ret;
    }

    ty_speaker_set_status(IPC_SPEAKER_STATUS_INIT);

    pthread_detach(phdl->stid);
    
    return 0;
}

/* pause speaker playing */
INT_T ty_speaker_pause_file()
{
    ty_speaker_set_pause(1);
    
    return 0;
}

/* stop speaker playing */
INT_T ty_speaker_stop_file()
{
    ty_speaker_set_stop(1);

    return 0;
}

/* set speaker volume */
INT_T ty_speaker_set_volume(INT_T volume)
{
    /* if user not use alsa interface, please use other audio volume set code instead. */
    return ty_alsa_set_volume(volume);
}
#endif
